#! /bin/bash
#
# MySQL tables dump using MySQL root user - call as Linux root user!
# Part of mysql-addons package. Original from Alex Hornyak (THX!)
#
# Usage: $0 [-c|--compress] database tablelist_file_pathname [backup_path|- [dump_user]
# All tables are dumped into one file!

EXIT_ERR="1"
USAGE="Usage: $0 [-c|--compress] database tablelist_file_pathname [backup_path|- [dump_user]]"

# Need to be root.. 
if [ ! $UID -eq 0 ]; then echo "Run as root, please." >&2; exit $EXIT_ERR; fi
# Getting parameters. 
# Optional compress switch (-c or --compress). 
option=`echo $1 | sed -n "s/^-\+//p"`; if [ ! -z "$option" ]; then shift; fi
if [ "$option" = "c" -o "$option" = "compress" ]; then.
    compress="/bin/gzip"
else.
    compress="cat"
fi
# Mandatory database name.
database=${1?"$USAGE"};shift
# Mandatory and not empty table list.
tablelist=${1?"$USAGE"};shift
tables=`cat $tablelist 2>/dev/null`
if [ -z "$tables" ]; then echo "No tables in $tablelist." >&2; exit $EXIT_ERR; fi
# Optional backup file pathname, defaults to ./hostname.timestamp.database.sql 
filename=${1:-`pwd`/`hostname`.`date +%Y%m%d%H%M%S`.$database.sql}; shift
if [ "$filename" = "-" ]; then
    # If '-' given as filename outpust to STDOUT. 
    filename=""
else
    # If compressed, adjusting filename to sql.gz. 
    case "${compress##*/}" in
        "gzip")
        filename="${filename%.gz}.gz"
        ;;
    esac
fi
# Optional Linux backups owner defaults to 'backup'. 
dumpuser=${1:-"backup"}; shift
# Wired access rights. 
dumpfilerights="640"
# MySQL command line call as MySQL root user. 
mysql_dsm="/usr/bin/mysql --defaults-file=/etc/mysql/root.cnf"
# MySQL dump command line call as MySQL root user. 
mysqldump_dsm="/usr/bin/mysqldump --defaults-file=/etc/mysql/root.cnf"
# MySQL dump parameters.
dumpparameters="--tables --complete-insert --add-drop-table --extended-insert"

# Checking database given. 
if [ -z "`$mysql_dsm --batch --execute='show databases;' | \ 
          sed s/^Database$// | \ 
          sed -n s/^$database$/X/p`" ]; then
    echo "MySQL database \"$database\" isn't found." >&2
    exit $EXIT_ERR
fi
# Doing some cleanup. 
if [ -f "$filename" ]; then rm "$filename"; fi
# Enumerating and dumping tables.
for table in $tables
do
    if [ ! -z "$filename" ]; then
        # Dump into a file.
        echo $database.$table
        $mysqldump_dsm $dumpparameters $database $table | $compress  >>"$filename"
    else
        # Dump top STDOUT.
        $mysqldump_dsm $dumpparameters $database $table | $compress
    fi
done
# Setting access rights on dump file.
if [ -f "$filename" ]; then 
    chown $dumpuser "$filename"; chmod $dumpfilerights "$filename"
fi
