#! /bin/bash
#
# MySQL repair broken tables using MySQL root user - call as Linux root user!
# Part of mysql-addons package.
#
# Usage: mysql_repair.sh 

GREP="/bin/grep"                        # grep command call
EXIT_ERR=1                              # Return code on error

# Need to be root.
if [ ! $UID -eq 0 ]; then echo "Run as root, please." >&2; exit $EXIT_ERR; fi

# MySQL command line call as MySQL root user.
mysql_root="/usr/bin/mysql --defaults-file=/etc/mysql/root.cnf"

# List of existing databases.
dblist=`$mysql_root --batch --execute='show databases;' | sed "s/^Database$//"`
if [ -n "$dblist" ]; then
    # Enumerating databases.
    for dbname in $dblist
    do
        # Information Scheme cannot be repaired by root.
        if [ "$dbname" != "information_schema" ]; then
            # Enumerating tables.
            tblist=`$mysql_root --batch --execute="use $dbname; show tables;" | sed "s/^Tables_in_$dbname//"`
            for tablename in $tblist
            do
                # Repair table for enumerated tables.
                result=`$mysql_root --batch --execute="repair table $dbname.$tablename;" | $GREP "$dbname.$tablename"`
                echo -e "$result"
            done
        fi
    done
fi
